-----------------------------------------------------------------------------

local SML = AceLibrary("SharedMedia-1.0")
local SML_font = SML:List("font")
local Dewdrop = AceLibrary:HasInstance("Dewdrop-2.0") and AceLibrary("Dewdrop-2.0")
-----------------------------------------------------------------------------

local AssessmentDisplaySets = Assessment:NewModule("DisplaySets", "AceEvent-2.0")
AssessmentDisplaySets.db = Assessment:AcquireDBNamespace("DisplaySets")
local profileDefaults =
{
	displaySets =
	{
		-- Defaults are inserted here in OnInitialize
	}
}

-----------------------------------------------------------------------------

local new, del, reset, copy = Assessment.NewTable, Assessment.ReleaseTable, Assessment.ResetTable, Assessment.CopyTable
local pairs = pairs

-----------------------------------------------------------------------------

local L = AceLibrary("AceLocale-2.2"):new("AssessmentDisplaySets")
L:RegisterTranslations("enUS", function() return {
	["Assessment"] = true,

	["Display Sets"] = true,
	["Display set options."] = true,

	["Add New Display Set"] = true,
	["Adds a new display set to the user interface."] = true,
	["Enter a new display set name"] = true,
	["Enter the name to copy this display set to"] = true,

	["Options for display set %s"] = true,

	["List Type"] = true,
	["Select a list type that controls what is displayed."] = true,
	["Bar Value Tag"] = true,
	["Select how to calculate bar values."] = true,
	["Normalize Bar Values"] = true,
	["When enabled, scales the maximum value to span the width of the display. When disabled, assumes that the bar value lies between 0 and 1."] = true,
	["Sort Key Tag"] = true,
	["Select how to sort the bars."] = true,
	["Descending Sort"] = true,
	["Enable to use descending sort. Disable to use ascending."] = true,

	["List Data Tag"] = true,
	["List data tag."] = true,
	["Parameters separated by \",\""] = true,

	["Favorite"] = true,
	["Show this display set as an option when clicking on a window."] = true,
	["Selectable"] = true,
	["Make this display set selectable from window menus."] = true,
	
	["Title Tag"] = true,
	["Select what to show for the window's title."] = true,

	["Filter Type"] = true,
	["Select how to filter the list."] = true,
	["Filter Tag"] = true,
	["Advanced filter type setting."] = true,

	["Fight Data"] = true,
	["Select which fight data set to display."] = true,
	["Inherit"] = true,
	["Total Data"] = true,
	["Current Fight"] = true,
	["Previous Fight"] = true,

	["Text"] = true,
	["Text options."] = true,
	["Add New Text"] = true,
	["Adds a new text to the display set."] = true,

	["Text %d"] = true,
	["Options for text %d"] = true,

	["Visible"] = true,
	["Marks this text as visible for display. Invisible text is only shown for reporting."] = true,

	["Text Tag"] = true,
	["Select how to display text."] = true,

	["Alignment"] = true,
	["Select how to align the text."] = true,
	["Left"] = true,
	["Center"] = true,
	["Right"] = true,

	["Offsets"] = true,
	["Set text offsets."] = true,
	["Left"] = true,
	["Set the left offset for the text."] = true,
	["Right"] = true,
	["Set the right offset for the text."] = true,
	["Vertical"] = true,
	["Set the vertical offset for the text."] = true,

	["Click Options"] = true,
	["Click options."] = true,
	["Action"] = true,
	["Specify what to do when a bar is clicked."] = true,
	["Ignore"] = true,
	["Show Display Set"] = true,
	["Display Set"] = true,
	["Select a display set to show when the bar is clicked."] = true,
	["Display Set Tag"] = true,
	["Advanced display set specification."] = true,
	["Data"] = true,
	["On click data."] = true,

	["Segment Options"] = true,
	["Segment options."] = true,
	["Enable"] = true,
	["Enable segment display."] = true,
	["Display Set"] = true,
	["Select a display set to use for segment data."] = true,
	["Data"] = true,
	["Segment data."] = true,

	["Default Color"] = true,
	["Change the default bar color."] = true,
	["Remove"] = true,
	["Remove this text."] = true,

	["Remove"] = true,
	["Remove this display set."] = true,
	["This display set cannot be removed currently because the display set %s references it for on-click."] = true,
	["This display set cannot be removed currently because the display set %s references it for segments."] = true,
	["Copy"] = true,
	["Copy this display set."] = true,
	
	["Any Tag String"] = true,
	["None"] = true,
	["Normal"] = true,
	["Thick"] = true,
	
	["Display Set List"] = true,

	["Color Type List"] = true,
	["Select what color types to use."] = true,
	["Comma separated color types. This is a temporary configuration measure. use /print Assessment.ColorType to get a list of valid names."] = true,
} end)

L:RegisterTranslations("koKR", function() return {
	["Assessment"] = "Assessment",

	["Display Sets"] = "디스플레이 세트",
	["Display set options."] = "디스플레이 세트 설정입니다.",

	["Add New Display Set"] = "새 디스플레이 세트 추가",
	["Adds a new display set to the user interface."] = "사용자 인터페이스에 새로운 디스플레이 세트를 추가합니다.",
	["Enter a new display set name"] = "새로운 디스플레이 세트의 이름을 입력하세요.",
	["Enter the name to copy this display set to"] = "해당 디스플레이 세트에 복사할 이름을 입력하세요.",

	["Options for display set %s"] = "디스플레이 세트 %s에 대한 설정",

	["List Type"] = "목록 유형",
	["Select a list type that controls what is displayed."] = "표시할 정보를 조절하는 목록 유형을 선택하세요.",
	["Bar Value Tag"] = "막대 수치",
	["Select how to calculate bar values."] = "막대 수치를 계산하기 위한 방법을 선택하세요.",
	["Normalize Bar Values"] = "막대 수치 표준화",
	["When enabled, scales the maximum value to span the width of the display. When disabled, assumes that the bar value lies between 0 and 1."] = "사용 시, 디스플레이 세트의 너비를 스팬하기 위한 최대 수치를 변경합니다. 미사용 시, 막대 수치를 0과 1사이 값으로 가정합니다.",
	["Sort Key Tag"] = "정렬 방법",
	["Select how to sort the bars."] = "막대를 정렬 할 방법을 선택하세요.",
	["Descending Sort"] = "내림차순 정렬",
	["Enable to use descending sort. Disable to use ascending."] = "내림차순 정렬을 사용합니다. 오름차순을 사용하려면 끄세요.",

	["List Data Tag"] = "목록 데이터",
	["List data tag."] = "목록 데이터 입니다.",
	["Parameters separated by \",\""] = "매개 변수는 \",\"로 구분 됩니다.",

	["Favorite"] = "즐겨찾기",
	["Show this display set as an option when clicking on a window."] = "창 클릭 시 설정에 따른 해당 디스플레이 세트를 표시합니다.",
	["Selectable"] = "선택 가능",
	["Make this display set selectable from window menus."] = "창 메뉴에서 해당 디스플레이 세트를 선택 할 수 있도록 합니다.",
	
	["Title Tag"] = "제목",
	["Select what to show for the window's title."] = "창의 제목에 표시할 정보를 선택합니다.",

	["Filter Type"] = "필터 종류",
	["Select how to filter the list."] = "목록 필터링 방법을 선택합니다.",
	["Filter Tag"] = "필터",
	["Advanced filter type setting."] = "고급 필터 유형 설정입니다.",

	["Fight Data"] = "전투 데이터",
	["Select which fight data set to display."] = "표시할 전투 데이터 세트를 선택하세요.",
	["Inherit"] = "상속",
	["Total Data"] = "전체 데이터",
	["Current Fight"] = "최근 전투",
	["Previous Fight"] = "이전 전투",

	["Text"] = "텍스트",
	["Text options."] = "텍스트 설정입니다.",
	["Add New Text"] = "새 텍스트 추가",
	["Adds a new text to the display set."] = "디스플레이 세트에 새로운 텍스트를 추가합니다.",

	["Text %d"] = "텍스트 %d",
	["Options for text %d"] = "텍스트 %d에 대한 설정",

	["Visible"] = "표시",
	["Marks this text as visible for display. Invisible text is only shown for reporting."] = "해당 텍스트를 표시 가능 하도록 합니다. 표시하지 않은 텍스트는 보고 시에만 표시됩니다.",

	["Text Tag"] = "텍스트 태그",
	["Select how to display text."] = "텍스트를 표시할 방법을 선택하세요.",

	["Alignment"] = "정렬",
	["Select how to align the text."] = "텍스트의 정렬 방법을 선택하세요.",
	["Left"] = "왼쪽",
	["Center"] = "가운데",
	["Right"] = "오른쪽",

	["Offsets"] = "오프셋",
	["Set text offsets."] = "텍스트의 오프셋을 설정합니다.",
	["Left"] = "왼쪽",
	["Set the left offset for the text."] = "텍스트의 왼쪽 오프셋을 설정합니다.",
	["Right"] = "오른쪽",
	["Set the right offset for the text."] = "텍스트의 오른쪽 오프셋을 설정합니다.",
	["Vertical"] = "수직",
	["Set the vertical offset for the text."] = "텍스트의 수직 오프셋을 설정합니다.",

	["Click Options"] = "클릭 설정",
	["Click options."] = "클릭 설정입니다.",
	["Action"] = "동작",
	["Specify what to do when a bar is clicked."] = "막대를 클릭했을 때의 동작을 지정합니다.",
	["Ignore"] = "제외",
	["Show Display Set"] = "디스플레이 세트 표시",
	["Display Set"] = "디스플레이 세트",
	["Select a display set to show when the bar is clicked."] = "막대를 클릭했을 때 표시할 디스플레이 세트를 선택하세요.",
	["Display Set Tag"] = "디스플레이 세트 태그",
	["Advanced display set specification."] = "고급 디스플레이 세트 설정입니다.",
	["Data"] = "데이터",
	["On click data."] = "On click data.",

	["Segment Options"] = "단편 설정",
	["Segment options."] = "단편 설정입니다.",
	["Enable"] = "사용",
	["Enable segment display."] = "단편 표시를 사용합니다.",
	["Display Set"] = "디스플레이 세트",
	["Select a display set to use for segment data."] = "단편 데이터를 사용 할 디스플레이 세트를 선택합니다.",
	["Data"] = "데이터",
	["Segment data."] = "단편 데이터입니다.",

	["Default Color"] = "기본 색상",
	["Change the default bar color."] = "기본 막대 색상을 변경합니다.",
	["Remove"] = "제거",
	["Remove this text."] = "해당 텍스트를 제거합니다.",

	["Remove"] = "제거",
	["Remove this display set."] = "해당 디스플레이 세트를 제거합니다.",
	["This display set cannot be removed currently because the display set %s references it for on-click."] = "현재 해당 디스플레이 세트는 on-click에서 %s 디스플레이 세트를 참조하기 때문에 제거할 수 없습니다.",
	["This display set cannot be removed currently because the display set %s references it for segments."] = "현재 해당 디스플레이 세트는 단편에서 %s 디스플레이 세트를 참조하기 때문에 제거할 수 없습니다.",
	["Copy"] = "복사",
	["Copy this display set."] = "해당 디스플레이 세트를 복사합니다.",
	
	["Any Tag String"] = "태그 문자열",
	["None"] = "없음",
	["Normal"] = "보통",
	["Thick"] = "두껍게",
	
	["Display Set List"] = "디스플레이 세트 목록",

	["Color Type List"] = "색상 종류 목록",
	["Select what color types to use."] = "사용할 색상 종류를 선택합니다.",
	["Comma separated color types. This is a temporary configuration measure. use /print Assessment.ColorType to get a list of valid names."] = "색상 종류는 콤마로 구분됩니다. 이것은 임시 설정입니다. /print Assessment를 사용하세요. 이용가능한 색상종류를 얻어옵니다.",
} end)

-----------------------------------------------------------------------------

Assessment.Options.args.uiSpacer =
{
	type = 'header',
	order = 50,
}

Assessment.Options.args.displaySets =
{
	type = "group",
	name = L["Display Sets"],
	desc = L["Display set options."],
	order = 60,
	args = { },
}

-----------------------------------------------------------------------------

local get = AssessmentDisplaySets.GetOptionValue
local set = AssessmentDisplaySets.SetOptionValue

local displaySetTagTypes =
{
	["barValueTag"]			= "number",
	["sortKeyTag"]			= "raw",
	["listDataTag"]			= "string",
	["titleTag"]			= "string",
	["filterTag"]			= "raw",
	["segmentDisplaySet"]	= "string",
	["segmentData"]			= "string",
	["onClickDisplaySet"]	= "string",
	["onClickData"]			= "string"
}

local function displaySetRawGet(passValue)
	return get(AssessmentDisplaySets.db.profile.displaySets[passValue[1]], passValue[2])
end
local function displaySetGet(passValue)
	local value = get(AssessmentDisplaySets.db.profile.displaySets[passValue[1]], passValue[2])
	local tagType = displaySetTagTypes[passValue[2]]
	if tagType then
		return Assessment:HighlightTagSyntax(value, tagType)
	end
	return value
end

local function displaySetSet(passValue, value)
	local tagType = displaySetTagTypes[passValue[2]]
	
	if tagType then
		value = Assessment:FixTagCasing(value, tagType)
	end

	set(AssessmentDisplaySets.db.profile.displaySets[passValue[1]], passValue[2], value)
	AssessmentDisplaySets:UpdateDisplaySets(passValue[1])
end

local function displaySetGetColorTypeList(passValue)
	local colorTypeList = AssessmentDisplaySets.db.profile.displaySets[passValue[1]].colorTypeList
	-- Clear out inappropriate entries
	for i=colorTypeList.count+1, #colorTypeList do
		colorTypeList[i] = nil
	end
	return table.concat(colorTypeList, ",")
end

local function displaySetSetColorTypeList(passValue, value)
	local colorTypeList = AssessmentDisplaySets.db.profile.displaySets[passValue[1]].colorTypeList
	if value == "" then
		reset(colorTypeList)
		colorTypeList.count = 0
		AssessmentDisplaySets:UpdateDisplaySets(passValue[1])
		return
	end
	
	local colorList = { (","):split(value) }
	
	for i=1,#colorList do
		if not Assessment.ColorType[colorList[i]] then
			Assessment:Print("%s is not a valid ColorType", colorList[i])
			return
		end
	end
	
	reset(colorTypeList)

	for i=1,#colorList do
		colorTypeList[i] = colorList[i]
	end
	
	colorTypeList.count = #colorList
	del(colorList)
	AssessmentDisplaySets:UpdateDisplaySets(passValue[1])
end

local function textGet(passValue)
	local value = get(AssessmentDisplaySets.db.profile.displaySets[passValue[1][1]].text[passValue[1][2]], passValue[2])
	if passValue[2] == "tag" then
	 	return Assessment:HighlightTagSyntax(value, "string")
	end
	return value
end

local function textSet(passValue, value)
	if passValue[2] == "tag" then
		value = Assessment:FixTagCasing(value, "string")
	end
	set(AssessmentDisplaySets.db.profile.displaySets[passValue[1][1]].text[passValue[1][2]], passValue[2], value)
	AssessmentDisplaySets:UpdateDisplaySets(passValue[1][1])
end

local function displaySetRemove(passValue)
	AssessmentDisplaySets:RemoveDisplaySet(passValue[1])
end

local function displaySetCopy(passValue)
	AssessmentDisplaySets:CopyDisplaySet(passValue[1])
end

-----------------------------------------------------------------------------

local fullDisplaySetValidate	= { }
local listTypeValidate			= { }
local filterTypeValidate		= { }

local displaySetsArgs =
{
	add =
	{
		type = "execute",
		name = L["Add New Display Set"],
		desc = L["Adds a new display set to the user interface."],
		func = function() AssessmentDisplaySets:AddDisplaySet() end,
		order = -1
	},
}

local defaultDisplaySetProfile =
{
	titleTag		= "Assessment",
	
	listType		= "GroupMembers", 
	barValueTag		= "DamageDealt", 
	sortKeyTag		= "BarValue",
	filterTag		= "1",
	descendingSort	= true,
	fightIndex		= 0,
--	colorTypeList	= { "Class", count = 1 },
	colorTypeList	= { count = 0 },
	defaultColor	= { 0.3, 0.45, 0.75 },
	favorite		= true,
	selectable		= true,
	listDataTag		= "",

	normalizeBarValues	= true,
	
	segmentEnable		= false,
	segmentDisplaySet	= "",
	segmentData			= "",
	
	onClickType			= "NONE",		-- NONE, or SHOW_DISPLAY_SET currently.
	onClickDisplaySet	= "",
	onClickData			= "",
	
	text =
	{
		count = 2,
		{
			visible		= true,
			alignment	= "LEFT",
			tag			= "[Rank]. [Key]",
			offsets =
			{
				left		= 0,
				right		= -20,
				vertical	= 0,
			},
		},
		{
			visible		= true,
			alignment	= "RIGHT",
			tag			= "[BarValue]",
			offsets =
			{
				left		= 0,
				right		= 0,
				vertical	= 0,
			},
		},
	},
}

local defaultTextProfile =
{
	visible		= true,
	alignment	= "LEFT",
	tag			= "text",
	offsets =
	{
		left		= 0,
		right		= 0,
		vertical	= 0,
	},
}

local displaySetOptionsTemplate =
{
	type = "group",
--	name = ["%s"] = true,
--	desc = ["Options for display set %s"] = true,
	args =
	{
		listType =
		{
			type = "text",
			name = L["List Type"],
			desc = L["Select a list type that controls what is displayed."],
			get = displaySetGet,
			set = displaySetSet,
			passValue = "listType",
			validate = listTypeValidate,
			order = 1,
		},
		barValueTag =
		{
			type = "text",
			name = L["Bar Value Tag"],
			desc = L["Select how to calculate bar values."],
			get = displaySetGet,
			set = displaySetSet,
			passValue = "barValueTag",
			usage = L["Any Tag String"],
		},
		normalizeBarValues =
		{
			type = "toggle",
			name = L["Normalize Bar Values"],
			desc = L["When enabled, scales the maximum value to span the width of the display. When disabled, assumes that the bar value lies between 0 and 1."],
			get = displaySetGet,
			set = displaySetSet,
			passValue = "normalizeBarValues",
		},
		colorTypeList =
		{
			type = "text",
			name = L["Color Type List"],
			desc = L["Select what color types to use."],
			get = displaySetGetColorTypeList,
			set = displaySetSetColorTypeList,
			passValue = false,
			usage = L["Comma separated color types. This is a temporary configuration measure. use /print Assessment.ColorType to get a list of valid names."],
		},
		sortKeyTag =
		{
			type = "text",
			name = L["Sort Key Tag"],
			desc = L["Select how to sort the bars."],
			get = displaySetGet,
			set = displaySetSet,
			passValue = "sortKeyTag",
			usage = L["Any Tag String"],
		},
		listDataTag =
		{
			type = "text",
			name = L["List Data Tag"],
			desc = L["List data tag."],
			get = displaySetGet,
			set = displaySetSet,
			passValue = "listDataTag",
			usage = L["Parameters separated by \",\""],
		},
		titleTag =
		{
			type = "text",
			name = L["Title Tag"],
			desc = L["Select what to show for the window's title."],
			get = displaySetGet,
			set = displaySetSet,
			passValue = "titleTag",
			usage = L["Any Tag String"],
		},
		filterType =
		{
			type = "text",
			name = L["Filter Type"],
			desc = L["Select how to filter the list."],
			get = displaySetRawGet,
			set = displaySetSet,
			passValue = "filterTag",
			validate = filterTypeValidate,
		},
		filterTag =
		{
			type = "text",
			name = L["Filter Tag"],
			desc = L["Advanced filter type setting."],
			get = displaySetGet,
			set = displaySetSet,
			passValue = "filterTag",
			usage = L["Any Tag String"],
		},
		descendingSort =
		{
			type = "toggle",
			name = L["Descending Sort"],
			desc = L["Enable to use descending sort. Disable to use ascending."],
			get = displaySetGet,
			set = displaySetSet,
			passValue = "descendingSort",
		},
		fightData =
		{
			type = "text",
			name = L["Fight Data"],
			desc = L["Select which fight data set to display."],
			get = function(passValue) return tostring( displaySetGet(passValue) ) end,
			set = function(passValue, value) displaySetSet(passValue, tonumber(value)) end,
			passValue = "fightIndex",
			validate = { ["-1"] = L["Inherit"], ["0"] = L["Total Data"], ["1"] = L["Current Fight"], ["2"] = L["Previous Fight"] },
		},
		segment =
		{
			type = "group",
			name = L["Segment Options"],
			desc = L["Segment options."],
			args =
			{
				enable =
				{
					type = "toggle",
					name = L["Enable"],
					desc = L["Enable segment display."],
					get = displaySetGet,
					set = displaySetSet,
					passValue = "segmentEnable",
					order = 1
				},
				displaySet =
				{
					type = "text",
					name = L["Display Set"],
					desc = L["Select a display set to use for segment data."],
					get = displaySetGet,
					set = displaySetSet,
					passValue = "segmentDisplaySet",
					validate = fullDisplaySetValidate,
				},
				data =
				{
					type = "text",
					name = L["Data"],
					desc = L["Segment data."],
					get = displaySetGet,
					set = displaySetSet,
					passValue = "segmentData",
					usage = L["Any Tag String"],
				},
			},
		},
		click =
		{
			type = "group",
			name = L["Click Options"],
			desc = L["Click options."],
			args =
			{
				action =
				{
					type = "text",
					name = L["Action"],
					desc = L["Specify what to do when a bar is clicked."],
					get = displaySetGet,
					set = displaySetSet,
					passValue = "onClickType",
					validate = { NONE = L["Ignore"], SHOW_DISPLAY_SET = L["Show Display Set"] }
				},
				displaySet =
				{
					type = "text",
					name = L["Display Set"],
					desc = L["Select a display set to show when the bar is clicked."],
					disabled = function(passValue) return AssessmentDisplaySets.db.profile.displaySets[passValue[1]].onClickType == "NONE" end,
					get = displaySetGet,
					set = displaySetSet,
					passValue = "onClickDisplaySet",
					validate = fullDisplaySetValidate,
				},
				displaySetTag =
				{
					type = "text",
					name = L["Display Set Tag"],
					desc = L["Advanced display set specification."],
					disabled = function(passValue) return AssessmentDisplaySets.db.profile.displaySets[passValue[1]].onClickType == "NONE" end,
					get = displaySetGet,
					set = displaySetSet,
					passValue = "onClickDisplaySet",
					usage = L["Any Tag String"],
				},
				data =
				{
					type = "text",
					name = L["Data"],
					desc = L["On click data."],
					disabled = function(passValue) return AssessmentDisplaySets.db.profile.displaySets[passValue[1]].onClickType == "NONE" end,
					get = displaySetGet,
					set = displaySetSet,
					passValue = "onClickData",
					usage = L["Any Tag String"],
				},
			},
		},
		defaultColor =
		{
			type = "color",
			name = L["Default Color"],
			desc = L["Change the default bar color."],
			get = function(passValue) return unpack(displaySetGet(passValue)) end,
			set = function(passValue, r, g, b, a) displaySetSet(passValue, { r, g, b, a }) end,
			passValue = "defaultColor",
			hasAlpha = true,
		},
		text =
		{
			type = "group",
			name = L["Text"],
			desc = L["Text options."],
			args =
			{
				add =
				{
					type = "execute",
					name = L["Add New Text"],
					desc = L["Adds a new text to the display set."],
					func = function(passValue) AssessmentDisplaySets:AddText(passValue[1]) end,
					passValue = true,
					order = -1
				},
			},
		},
		favorite =
		{
			type = "toggle",
			name = L["Favorite"],
			desc = L["Show this display set as an option when clicking on a window."],
			disabled = function(passValue) return not AssessmentDisplaySets.db.profile.displaySets[passValue[1]].selectable end,
			get = displaySetGet,
			set = function(passValue, v) displaySetSet(passValue, v) AssessmentDisplaySets:UpdateDisplaySetLists() end,
			passValue = "favorite",
		},
		selectable =
		{
			type = "toggle",
			name = L["Selectable"],
			desc = L["Make this display set selectable from window menus."],
			get = displaySetGet,
			set = function(passValue, v) displaySetSet(passValue, v) AssessmentDisplaySets:UpdateDisplaySetLists() end,
			passValue = "selectable",
		},
		copy =
		{
			type = "execute",
			name = L["Copy"],
			desc = L["Copy this display set."],
			func = displaySetCopy,
			passValue = false,
			order = -2,
		},
		remove =
		{
			type = "execute",
			name = L["Remove"],
			desc = L["Remove this display set."],
			func = displaySetRemove,
			passValue = false,
			order = -1,
		},
	}
}

local textOptionsTemplate =
{
	type = "group",
--	name = ["Text %d"] = true,
--	desc = ["Options for text %d"] = true,
	args =
	{
		tag =
		{
			type = "text",
			name = L["Text Tag"],
			desc = L["Select how to display text."],
			get = textGet,
			set = textSet,
			passValue = "tag",
			usage = L["Any Tag String"],
		},
		alignment =
		{
			type = "text",
			name = L["Alignment"],
			desc = L["Select how to align the text."],
			get = textGet,
			set = textSet,
			passValue = "alignment",
			validate = { LEFT = L["Left"], CENTER = L["Center"], RIGHT = L["Right"] },
		},
		visible =
		{
			type = "toggle",
			name = L["Visible"],
			desc = L["Marks this text as visible for display. Invisible text is only shown for reporting."],
			get = textGet,
			set = textSet,
			passValue = "visible",
		},
		offsets =
		{
			type = "group",
			name = L["Offsets"],
			desc = L["Set text offsets."],
			args =
			{
				left =
				{
					type = "range",
					name = L["Left"],
					desc = L["Set the left offset for the text."],
					min = -100,
					max = 100,
					step = 1,
					get = textGet,
					set = textSet,
					passValue = "offsets.left",
				},
				right =
				{
					type = "range",
					name = L["Right"],
					desc = L["Set the right offset for the text."],
					min = -100,
					max = 100,
					step = 1,
					get = textGet,
					set = textSet,
					passValue = "offsets.right",
				},
				vertical =
				{
					type = "range",
					name = L["Vertical"],
					desc = L["Set the vertical offset for the text."],
					min = -32,
					max = 32,
					step = 1,
					get = textGet,
					set = textSet,
					passValue = "offsets.vertical",
				},
			}
		},
		remove =
		{
			type = "execute",
			name = L["Remove"],
			desc = L["Remove this text."],
			func = function(passValue) AssessmentDisplaySets:RemoveText(unpack(passValue[1])) end,
			passValue = false,
			order = -1,
		},
	}
}

-----------------------------------------------------------------------------

Assessment.DefaultDisplaySets.Assessment =
{
	titleTag			= "Assessment",
	listType			= "DisplaySetList",
	barValueTag			= "0", 
	filterTag			= "Key ~= \""..L["Assessment"].."\" & Favorite & Selectable",
	fightIndex			= 0,
	text				= { count = 1, { alignment = "CENTER", tag = "[Key:Cyan]", offsets = { right = 0 } } },
	defaultColor		= { 0.3, 0.45, 0.75 },
	description			= L["Assessment"],
	onClickType			= "SHOW_DISPLAY_SET",
	onClickDisplaySet	= "[Key]",
	onClickData			= "",
}

Assessment.ListType.DisplaySetList =
{
	method = function() 
				return pairs(AssessmentDisplaySets.db.profile.displaySets)
			 end,
	description = L["Display Set List"],
	events = { "AssessmentDisplaySets:OnDisplaySetAdded", "AssessmentDisplaySets:OnDisplaySetRemoved", "AssessmentDisplaySets:OnDisplaySetListChanged" },
}

Assessment.TagData.Favorite =
{
	method = function(data)
				local displaySet = AssessmentDisplaySets.db.profile.displaySets[data.key]
				return displaySet and displaySet.favorite
			 end
}

Assessment.TagData.Selectable =
{
	method = function(data)
				local displaySet = AssessmentDisplaySets.db.profile.displaySets[data.key]
				return displaySet and displaySet.selectable
			 end
}

-----------------------------------------------------------------------------

function AssessmentDisplaySets:CreateValidateLists()
    for k,v in pairs(Assessment.ListType) do
        listTypeValidate[k] = v.description
    end 
            
    for k,v in pairs(Assessment.FilterType) do
        filterTypeValidate[v.tag] = v.description
    end     
end

-----------------------------------------------------------------------------

local function MergeTable(destination, source)
	for k,v in pairs(source) do
		if type(v) == "table" then
			if not destination[k] then
				destination[k] = new()
			end
			MergeTable(destination[k], v)
		else
			destination[k] = v
		end
	end
end

local function MergeTableIfEmpty(destination, source)
	for k,v in pairs(source) do
		if type(v) == "table" then
			if not destination[k] then
				destination[k] = new()
			end
			MergeTableIfEmpty(destination[k], v)
		elseif type(destination[k]) == "nil" then
			destination[k] = v
		end
	end
end

function AssessmentDisplaySets:OnInitialize()
	for k,v in pairs(Assessment.DefaultDisplaySets) do
		local displaySet = copy(defaultDisplaySetProfile)
		MergeTable(displaySet, v)
		for i=3,displaySet.text.count do
			MergeTableIfEmpty(displaySet.text[i], defaultTextProfile)
		end
		
		-- Cleanup fields
		displaySet.description = nil
		
		profileDefaults.displaySets[v.description] = displaySet
	end
	
	Assessment:RegisterDefaults("DisplaySets", "profile", profileDefaults)
	
	self:CreateValidateLists()
end

function AssessmentDisplaySets:OnEnable()
	self:CreateOptions()
	self:TriggerEvent("AssessmentDisplaySets:OnDisplaySetListChanged")
end

function AssessmentDisplaySets:CreateTextOptions(passValueData)
	local options = self:CopyOptionsTemplate(textOptionsTemplate, passValueData)
	options.name = L["Text %d"]:format(passValueData[2])
	options.desc = L["Options for text %d"]:format(passValueData[2])
	return options
end

function AssessmentDisplaySets:UpdateDisplaySetValidates()
	reset(fullDisplaySetValidate)

	for k, v in pairs(self.db.profile.displaySets) do
		fullDisplaySetValidate[k] = k
	end
end

function AssessmentDisplaySets:CreateOptions()
	self:UpdateDisplaySetValidates()

	del(Assessment.Options.args.displaySets.args, true)
	Assessment.Options.args.displaySets.args = copy(displaySetsArgs)
	
	for k in pairs(self.db.profile.displaySets) do
		if Assessment.Options.args.displaySets.args[k] then
			del(Assessment.Options.args.displaySets.args[k], true)
		end
		Assessment.Options.args.displaySets.args[k] = self:CreateDisplaySetOptions(k)
	end
end

function AssessmentDisplaySets:CreateDisplaySetOptions(displaySetName)
	local options = self:CopyOptionsTemplate(displaySetOptionsTemplate, displaySetName)
	options.name = displaySetName
	options.desc = L["Options for display set %s"]:format(displaySetName)

	for i = 1, self.db.profile.displaySets[displaySetName].text.count do
		-- We make it a string, otherwise AceConsole can't use it.....
		if options.args.text.args[tostring(i)] then
			del(options.args.text.args[tostring(i)], true)
		end
		options.args.text.args[tostring(i)] = self:CreateTextOptions( { displaySetName, i } )
	end
	
	return options
end

-----------------------------------------------------------------------------

StaticPopupDialogs["ASSESSMENT_ADD_DISPLAY_SET"] = 
{
	text = L["Enter a new display set name"],
	button1 = ACCEPT,
	button2 = CANCEL,
	hasEditBox = 1,
	OnAccept = function()
		local text = getglobal(this:GetParent():GetName().."EditBox"):GetText()
		AssessmentDisplaySets:AddDisplaySet(text)
	end,
	OnShow = function()
		getglobal(this:GetName().."EditBox"):SetFocus()
	end,
	EditBoxOnEnterPressed = function()
		local text = getglobal(this:GetParent():GetName().."EditBox"):GetText()
		AssessmentDisplaySets:AddDisplaySet(text)
		this:GetParent():Hide()
	end,
	timeout = 0,
	exclusive = 1,
	whileDead = 1,
	hideOnEscape = 1,
}

StaticPopupDialogs["ASSESSMENT_COPY_DISPLAY_SET"] = 
{
	text = L["Enter the name to copy this display set to"],
	button1 = ACCEPT,
	button2 = CANCEL,
	hasEditBox = 1,
	OnAccept = function()
		local text = getglobal(this:GetParent():GetName().."EditBox"):GetText()
		AssessmentDisplaySets:CopyDisplaySet(StaticPopupDialogs["ASSESSMENT_COPY_DISPLAY_SET"].data, text)
	end,
	OnShow = function()
		getglobal(this:GetName().."EditBox"):SetFocus()
	end,
	EditBoxOnEnterPressed = function()
		local text = getglobal(this:GetParent():GetName().."EditBox"):GetText()
		AssessmentDisplaySets:CopyDisplaySet(StaticPopupDialogs["ASSESSMENT_COPY_DISPLAY_SET"].data, text)
		this:GetParent():Hide()
	end,
	timeout = 0,
	exclusive = 1,
	whileDead = 1,
	hideOnEscape = 1,
}

-----------------------------------------------------------------------------

function AssessmentDisplaySets:UpdateDisplaySetLists()
	-- Called when favorite or selectable are changed
	self:TriggerEvent("AssessmentDisplaySets:OnDisplaySetListChanged", displaySetName)
end

function AssessmentDisplaySets:UpdateDisplaySets(displaySetName)
	self:TriggerEvent("AssessmentDisplaySets:OnDisplaySetUpdated", displaySetName)
end

function AssessmentDisplaySets:AddDisplaySet(displaySetName)
	if displaySetName then
		if not self.db.profile.displaySets[displaySetName] then
			self.db.profile.displaySets[displaySetName] = copy(defaultDisplaySetProfile)
			self:CreateOptions()
			self:TriggerEvent("AssessmentDisplaySets:OnDisplaySetAdded", displaySetName)
		end
	else
		if Dewdrop then Dewdrop:Close() end
		StaticPopup_Show("ASSESSMENT_ADD_DISPLAY_SET")
	end
end

function AssessmentDisplaySets:CopyDisplaySet(sourceSetName, newDisplaySetName)
	if newDisplaySetName then
		if not self.db.profile.displaySets[newDisplaySetName] then
			self.db.profile.displaySets[newDisplaySetName] = copy(self.db.profile.displaySets[sourceSetName])
			self:CreateOptions()
			self:TriggerEvent("AssessmentDisplaySets:OnDisplaySetAdded", newDisplaySetName)
		end
	else
		if Dewdrop then Dewdrop:Close() end
		StaticPopupDialogs["ASSESSMENT_COPY_DISPLAY_SET"].data = sourceSetName
		StaticPopup_Show("ASSESSMENT_COPY_DISPLAY_SET")
	end
end

local removeSetFailReason
function AssessmentDisplaySets:RemoveDisplaySet(displaySetName)
	for k,v in pairs(self.db.profile.displaySets) do

		if v.onClickType == "SHOW_DISPLAY_SET" and v.onClickDisplaySet == displaySetName then
			Assessment:Print(string.format(L["This display set cannot be removed currently because the display set %s references it for on-click."], k))
			return
		end

		if v.segmentEnable and v.segmentDisplaySet == displaySetName then
			Assessment:Print(string.format(L["This display set cannot be removed currently because the display set %s references it for segments."], k))
			return
		end
	end

	removeSetFailReason = nil
	self:TriggerEvent("AssessmentDisplaySets:OnDisplaySetRequestRemove", displaySetName)

	if removeSetFailReason then
		Assessment:Print(removeSetFailReason)
	else
		self.db.profile.displaySets[displaySetName] = nil
		self:CreateOptions()
		self:TriggerEvent("AssessmentDisplaySets:OnDisplaySetRemoved", displaySetName)
	end
end

function AssessmentDisplaySets:PreventDisplaySetRemoval(reason)
	removeSetFailReason = reason
end

function AssessmentDisplaySets:GetDisplaySet(displaySetName)
	return self.db.profile.displaySets[displaySetName]
end

function AssessmentDisplaySets:IterateDisplaySets()
	return pairs(self.db.profile.displaySets)
end

-----------------------------------------------------------------------------

function AssessmentDisplaySets:AddText(displaySetName)
	local text = self.db.profile.displaySets[displaySetName].text
	text.count = text.count + 1
	if not text[text.count] then
		text[text.count] = copy(defaultTextProfile)
	end

	self:UpdateDisplaySets(displaySetName)
	self:CreateOptions()
end

function AssessmentDisplaySets:RemoveText(displaySetName, removeIndex)
	local text = self.db.profile.displaySets[displaySetName].text
	
	for i = removeIndex, text.count do
		text[i] = text[i+1]
	end
	
	text.count = text.count-1

	self:UpdateDisplaySets(displaySetName)
	self:CreateOptions()
end

-----------------------------------------------------------------------------
